@@ -4,8 +4,9 @@ use strict;
 use warnings;
 
 BEGIN {
-    our $VERSION        = 0.04;
-    my @common          = qw[ FCGI_MAX_LENGTH
+    our $VERSION        = 0.05;
+    my @common          = qw[ FCGI_MAX_CONTENT_LEN
+                              FCGI_MAX_LEN
                               FCGI_HEADER_LEN
                               FCGI_VERSION_1
                               FCGI_NULL_REQUEST_ID ];
@@ -43,13 +44,19 @@ BEGIN {
                               FCGI_EndRequestBody
                               FCGI_UnknownTypeBody ];
 
+    my @name            = qw[ @FCGI_TYPE_NAME
+                              @FCGI_RECORD_NAME
+                              @FCGI_ROLE_NAME
+                              @FCGI_PROTOCOL_STATUS_NAME ];
+
     our @EXPORT_OK      = (  @common,
                              @type,
                              @role,
                              @flag,
                              @protocol_status,
                              @value,
-                             @pack );
+                             @pack,
+                             @name );
 
     our %EXPORT_TAGS =    (  all             => \@EXPORT_OK,
                              common          => \@common,
@@ -60,12 +67,71 @@ BEGIN {
                              value           => \@value,
                              pack            => \@pack );
 
+    our @FCGI_TYPE_NAME = (
+        undef,                        #  0
+        'FCGI_BEGIN_REQUEST',         #  1
+        'FCGI_ABORT_REQUEST',         #  2
+        'FCGI_END_REQUEST',           #  3
+        'FCGI_PARAMS',                #  4
+        'FCGI_STDIN',                 #  5
+        'FCGI_STDOUT',                #  6
+        'FCGI_STDERR',                #  7
+        'FCGI_DATA',                  #  8
+        'FCGI_GET_VALUES',            #  9
+        'FCGI_GET_VALUES_RESULT',     # 10
+        'FCGI_UNKNOWN_TYPE'           # 11
+    );
+
+    our @FCGI_RECORD_NAME = (
+        undef,                        #  0
+        'FCGI_BeginRequestRecord',    #  1
+        'FCGI_AbortRequestRecord',    #  2
+        'FCGI_EndRequestRecord',      #  3
+        'FCGI_ParamsRecord',          #  4
+        'FCGI_StdinRecord',           #  6
+        'FCGI_StdoutRecord',          #  6
+        'FCGI_StderrRecord',          #  7
+        'FCGI_DataRecord',            #  8
+        'FCGI_GetValuesRecord',       #  9
+        'FCGI_GetValuesResultRecord', # 10
+        'FCGI_UnknownTypeRecord',     # 11
+    );
+
+    our @FCGI_ROLE_NAME = (
+        undef,                        #  0
+        'FCGI_RESPONDER',             #  1
+        'FCGI_AUTHORIZER',            #  2
+        'FCGI_FILTER',                #  3
+    );
+
+    our @FCGI_PROTOCOL_STATUS_NAME = (
+        'FCGI_REQUEST_COMPLETE',      #  0
+        'FCGI_CANT_MPX_CONN',         #  1
+        'FCGI_OVERLOADED',            #  2
+        'FCGI_UNKNOWN_ROLE',          #  3
+    );
+
+    if (Internals->can('SvREADONLY')) { # 5.8
+        Internals::SvREADONLY(@FCGI_TYPE_NAME, 1);
+        Internals::SvREADONLY(@FCGI_RECORD_NAME, 1);
+        Internals::SvREADONLY(@FCGI_ROLE_NAME, 1);
+        Internals::SvREADONLY(@FCGI_PROTOCOL_STATUS_NAME, 1);
+        Internals::SvREADONLY($_, 1) for @FCGI_TYPE_NAME,
+                                         @FCGI_RECORD_NAME,
+                                         @FCGI_ROLE_NAME,
+                                         @FCGI_PROTOCOL_STATUS_NAME;
+    }
+
     require Exporter;
     *import = \&Exporter::import;
 }
 
+
+sub FCGI_LISTENSOCK_FILENO   () {      0 }
+
 # common
-sub FCGI_MAX_LENGTH          () { 0xFFFF }
+sub FCGI_MAX_CONTENT_LEN     () { 0xFFFF }
+sub FCGI_MAX_LEN             () { 0xFFFF } # deprecated
 sub FCGI_HEADER_LEN          () {      8 }
 sub FCGI_VERSION_1           () {      1 }
 sub FCGI_NULL_REQUEST_ID     () {      0 }
@@ -110,3 +176,4 @@ sub FCGI_EndRequestBody      () { 'NCx3'   }
 sub FCGI_UnknownTypeBody     () { 'Cx7'    }
 
 1;
+
